unit Dbsearch;
interface
uses SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
Forms, DB, DBTables, DsgnIntf,
Dbslocat, Sercheng, Pikflds;

type

   EDBSearchPatternNotFound = class(Exception);

type
   TdbSearch = class(TComponent)
   private
     FLocateInfo: TLocateInfo;
     FDataSource: TDataSource;
     FDefSearchField: String;
     FHelpFile: String;
     FSearchFields: TStringList;
     function GetDataSource: TDataSource;
     procedure SetDataSource(ds: TDataSource);
     procedure SetSearchFields(FieldNames: TStringList);
     procedure InitLocateInfo;
     function SearchInMemo(Blob: TBlobField; Pattern: PChar; CaseSensitive, ExactMatch: Boolean): Boolean;
     procedure SetMultiPattern(MultiPattern: string);
     function GetMultiPattern: string;
     procedure SetSinglePattern(SinglePattern: Char);
     function GetSinglePattern: Char;
   protected
     procedure DoLocate(LocateInfo: TLocateInfo; FromBeginning: Boolean);
   public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     function GetLocateInfo: boolean;
     procedure Locate;
     procedure LocateNext;
   published
     property DataSource: TDataSource read GetDataSource write SetDataSource;
     property DefSearchField: string read FDefSearchField write FDefSearchField;
     property SearchFields: TStringList read FSearchFields write SetSearchFields;
{**********  Aaron *********************}
     property MultiCharPattern: string read GetMultiPattern write SetMultiPattern;
     property SingleCharPattern: Char read GetSinglePattern write SetSinglePattern;
{**********  Aaron *********************}

   end;

  TDefSrchFldProperty = class(TStringProperty)
    procedure GetValues(Proc: TGetStrProc); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  procedure Register;


implementation
procedure Register;
begin
  RegisterComponents( 'Samples', [TdbSearch] );
  RegisterComponentEditor(TdbSearch, TdbSearchEditor);
  RegisterPropertyEditor( TypeInfo( TStringList ), TdbSearch, 'SearchFields', TSrchFldsProperty );
  RegisterPropertyEditor( TypeInfo(string), TdbSearch, 'DefSearchField', TDefSrchFldProperty );

end;

constructor TdbSearch.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSearchFields := TStringList.Create;
end;

destructor TdbSearch.Destroy;
begin
  inherited Destroy;
end;

{**********  Aaron *********************}
procedure TdbSearch.SetMultiPattern(MultiPattern: string);
var
   Temp: PChar;
begin
   GetMem(Temp, Length(MultiPattern) + 3);
   StrPCopy(Temp, MultiPattern);
   SetMultiPat(Temp);
   FreeMem(Temp, Length(MultiPattern) + 3);
end;

function TdbSearch.GetMultiPattern: string;
begin
   Result := StrPas(GetMultiPat);
end;

procedure TdbSearch.SetSinglePattern(SinglePattern: Char);
begin
   SetSinglePat(SinglePattern);
end;

function TdbSearch.GetSinglePattern: Char;
begin
   Result := GetSinglePat;
end;

{**********  Aaron *********************}

function TdbSearch.GetDataSource: TDataSource;
begin
  Result := FDataSource;
end;

procedure TdbSearch.SetDataSource(ds: TDatasource);
begin
  FDataSource := ds;
  InitLocateInfo;
end;

procedure TdbSearch.SetSearchFields(FieldNames: TStringList);
begin
  FSearchFields.Assign(FieldNames);
end;

procedure TdbSearch.InitLocateInfo;
begin
  with FLocateInfo do
  begin
    LocateText:='';
    CaseSensitive:=FALSE;
    ExactMatch:=FALSE;
  end;
end;

function TdbSearch.GetLocateInfo: Boolean;
var
  loopctr: integer;
begin
   Result:=False;
   DLGLocate := TDLGLocate.Create(Application);
   DLGLocate.rbPatternMatch.Caption := SingleCharPattern + ' &and ' + MultiCharPattern;
   try
   begin
     if (FDataSource <> nil) then
     begin
       if FSearchFields.Count > 0 then
       begin
         for loopctr := 0 to FSearchFields.Count -1 do
           DLGLocate.AddLocateField(SearchFields[loopctr]);
       end
       else
         {Search is permitted on all table fields}
         for loopctr:=0 to FDataSource.DataSet.FieldCount - 1 do
           {Add only the ones the end-user can see}
           if(FDataSource.DataSet.Fields[loopctr].Visible)then
            DLGLocate.AddLocateField(FDataSource.DataSet.Fields[loopctr].DisplayLabel);
       {Initialize the dialogbox fields}
       FLocateInfo.LocateField := FDefSearchField;
       DLGLocate.SetLocateInfo(FLocateInfo);
       {Show locate dialog}
       if(DLGLocate.ShowModal=IDOK)then
       begin
        {Get the results}
        FLocateInfo:=DLGLocate.GetLocateInfo;
        Result:=True;
       end;
     end;
   end
   finally
     DLGLocate.Free;
   end;
end;



procedure TdbSearch.DoLocate(LocateInfo: TLocateInfo; FromBeginning:
Boolean);

function CanSearchOnIndex(FieldName: String): Boolean;
var
  I: Integer;
begin
  Result := False;
  with (FDataSource.Dataset as TTable) do
  begin
    IndexDefs.Update;
    for I := 0 to IndexDefs.Count - 1 do
      if Pos(IndexDefs.Items[I].Fields, LocateInfo.LocateField) = 1 then
        Result := True;
  end;
end;

var
   Target, Pattern: Array[0..255] of Char;
   loopctr: Integer;
   oldpos: TBookmark;
   FieldNumber: Integer; {The index of the field that we are searching on}
   OldCursor: HCursor;
   StringMatch: Boolean;
   ActiveIndexFields: TStringList;
   ActiveIndex: String;
   I: Integer;
begin
  StrPCopy(Pattern, LocateInfo.LocateText);
  {Get the index of the field to search}
  for loopctr:=0 to FDataSource.Dataset.FieldCount - 1 do
    if FDataSource.Dataset.Fields[loopctr].DisplayLabel=LocateInfo.LocateField then
    begin
      FieldNumber:=loopctr;
      Break;
    end;
  if FieldNumber >= FDataSource.Dataset.FieldCount then
    raise EDBSearchPatternNotFound.Create('Search Field not in table');
  if (LocateInfo.ExactMatch or not IsPatternWild(Pattern)) and
    (FDataSource.Dataset.ClassName = 'TTable') and not
    (FDatasource.Dataset as TTable).Database.IsSQLBased and
    CanSearchOnIndex(LocateInfo.LocateText) then
  With (FDataSource.Dataset as TTable) do
  begin
    DisableControls;
    {Store the name of the currently active index}
    ActiveIndex := IndexName;
    ActiveIndexFields := TStringList.Create;
    try
      for I := 0 to IndexFieldCount - 1 do
        ActiveIndexFields.Add(IndexFields[I].FieldName);
      {Delphi will pick an index based on the columns specified in IndexFieldNames}
      IndexFieldNames := Fields[FieldNumber].FieldName;
      FindNearest([LocateInfo.LocateText]);
      {Replace field names with values}
      for I := 0 to ActiveIndexFields.Count - 1 do
        ActiveIndexFields[I]:= FieldByName(ActiveIndexFields[I]).AsString;
      {Get back to the original index and find the same record}
      IndexName := ActiveIndex;
      SetKey;
      for I := 0 to ActiveIndexFields.Count - 1 do
        IndexFields[I].AsString := ActiveIndexFields[I];
      GoToKey;
      Exit;
    finally
      ActiveIndexFields.Free;
      EnableControls;
    end;
  end;
  with FDataSource.DataSet do
  begin
    OldCursor:=SetCursor(LoadCursor(0, IDC_WAIT));
    DisableControls;
    {Save oldposition}
    oldpos:=GetBookmark;
    {Start from first record or the next one}
    if(FromBeginning)then
      First
    else
      Next;
    {Get and compare data}
    StringMatch := False;
    repeat
      if(Fields[FieldNumber].DataType = ftMemo) then
        StringMatch := SearchInMemo(TMemoField(Fields[FieldNumber]), Pattern,
                                    LocateInfo.CaseSensitive, not LocateInfo.ExactMatch)
      else
        StringMatch:=CompareStrings(StrPCopy(Target, Fields[FieldNumber].Text),
                                    Pattern, LocateInfo.CaseSensitive, not LocateInfo.ExactMatch);
      if StringMatch then
        Break
      else
        Next;
    until EOF;
    {enable controls}
    EnableControls;
    {End of the data and no match? reposition to previous position}
    if(not StringMatch)then
      Gotobookmark(oldpos);
    Freebookmark(oldpos);
  end;
  SetCursor(OldCursor);
  if(not StringMatch)then
      raise EDBSearchPatternNotFound.Create('Search pattern not found');
end;

function TdbSearch.SearchInMemo(Blob: TBlobField; Pattern: PChar; CaseSensitive, ExactMatch: Boolean): Boolean;
var
   BlobStream: TBlobStream;
   Buffer: Array[0..255] of Char;
   BytesRead: Byte;
   TotalRead: Word;
   P: PChar;
begin
{
   What follows is convoluted, I agree. I was forced to resort to it since
   I just could not get the BlobStream.Read method to read data into
   a GetMemED PChar. Creating a large character array on the stack did not
   appear to be a good idea.
}
   result := False;
   BlobStream := TBlobStream.Create(Blob, bmRead);
   {The largest single block that you can safely allocate on the heap is
65528}
   GetMem(P, 65528);
   try
     BytesRead := 255;
     TotalRead := 0;
     repeat
       BytesRead := BlobStream.Read(Buffer, 255);
       Buffer[BytesRead] := Chr(0);
       StrCopy(P+TotalRead, Buffer);
       TotalRead := TotalRead + BytesRead;
     until (BytesRead < 255) or (TotalRead > 65273);
     (P + TotalRead)^ := Chr(0);
     Result := CompareStrings(P, Pattern, CaseSensitive, ExactMatch);
   finally
      BlobStream.Free;
      FreeMem(P, 65528);
   end;
end;

procedure TdbSearch.Locate;
begin
  InitLocateInfo;
  {Get info from dialog then do search}
  if(GetLocateInfo=True)then
    DoLocate(FLocateInfo, True);
end;

procedure TdbSearch.LocateNext;
begin
   {Has a locate been done before?}
   if(FLocateInfo.LocateText='')then
     Locate
   else
      begin
      {Locate from the current position}
      DoLocate(FLocateInfo, False);
      end;
end;


{------ Default Search Field ------------}

function TDefSrchFldProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paSortList];
end;  {GetAttributes}


procedure TDefSrchFldProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  TheComponent: TdbSearch;
begin
  TheComponent := GetComponent(0) as TdbSearch;
  if TheComponent.DataSource <> nil then
  begin
{************* Aaron *****************}
      if(TheComponent.SearchFields.Count > 0)then
         begin
         for I := 0 to TheComponent.SearchFields.Count - 1 do
            Proc(TheComponent.SearchFields[I]);
         end
      else
         begin
         for I := 0 to (TheComponent.DataSource.DataSet.FieldCount - 1) do
            Proc(TheComponent.DataSource.DataSet.Fields[I].DisplayLabel);
         end;
{************* Aaron  *****************}
  end;
end;
end.

(*
// ==============================================================
// Copyright (c) 1996 by Aaron Castro and VK Thakur. All rights reserved
// Version 1.00 (Beta)   Date: 01/01/96
*)

