unit Dbslocat;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, db, DBTables, Buttons;


{************************************************************************}
{structure used to set and retrieve information from the locate dialogbox}
{************************************************************************}
type
   TLocateInfo = record
      LocateField: string;   {The field to search}
      LocateText: string;    {The text with pattern to search for}
      CaseSensitive: boolean;     {Is the search case sensitive?}
      ExactMatch: boolean;        {Should we look for an exact match?}
   end;


{************************************************************************}
{Locate Dialog box class}
{************************************************************************}
type
  TDLGLocate = class(TForm)

    GroupBox1: TGroupBox;
    txtSearchString: TEdit;
    GroupBox2: TGroupBox;
    Bevel1: TBevel;
    cbxCaseSensitive: TCheckBox;
    rbExactMatch: TRadioButton;
    rbPatternMatch: TRadioButton;
    GroupBox3: TGroupBox;
    cbxFields: TComboBox;
    bOK: TButton;
    procedure bOKClick(Sender: TObject);
    procedure bCancelClick(Sender: TObject);
    procedure txtSearchStringChange(Sender: TObject);

    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure bHelpClick(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
    HelpFile: string;
    procedure SetLocateInfo(Info: TLocateInfo);
    function GetLocateInfo: TLocateInfo;
    procedure AddLocateField(FieldName: string);
  end;


var
  DLGLocate: TDLGLocate;


implementation

{$R *.DFM}

function TDLGLocate.GetLocateInfo: TLocateInfo;
begin
  GetLocateInfo.LocateField:=cbxFields.Text;
  GetLocateInfo.LocateText:=txtSearchString.Text;
  GetLocateInfo.ExactMatch:=(rbExactMatch.Checked);
  GetLocateInfo.CaseSensitive:=(cbxCaseSensitive.State=cbChecked);
end;

procedure TDLGLocate.SetLocateInfo(Info: TLocateInfo);
begin
  {Fill in Field to search}
  with cbxFields do
    if(Info.LocateField<>'')then
      ItemIndex := Items.IndexOf(info.LocateField)
    else
      ItemIndex := 0;
  {Fill in locate text}
  TxtSearchString.Text:=Info.LocateText;
  {Exact match?}
  if(Info.ExactMatch)then
    rbExactMatch.Checked:=True
  else
    rbPatternMatch.Checked:=True;

  {Case sensitive?}
  if(Info.CaseSensitive)then
    cbxCaseSensitive.State:=cbChecked
  else
    cbxCaseSensitive.State:=cbUnchecked;
end;



{************************************************************************}
{
    AddLocateField
         FieldName (Input): Field to be added to searchable field list


     Adds fields to the combobox listing fields that can be searched
}
{************************************************************************}

procedure TDLGLocate.AddLocateField(FieldName: string);
begin
    cbxFields.Items.Add(FieldName);
end;



procedure TDLGLocate.bOKClick(Sender: TObject);
begin
    ModalResult:=mrOK;
end;


procedure TDLGLocate.bCancelClick(Sender: TObject);
begin
    ModalResult:=mrCancel;
end;

procedure TDLGLocate.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
const
   Egg: String = '';
begin
   if((ssAlt in Shift) and (ssCtrl in Shift))then
      begin
      case length(Egg) of
         0,1:if(Key=ord('A'))then Egg:=Egg+'A';
         2:if(Key=ord('R'))then Egg:=Egg+'R';
         3:if(Key=ord('O'))then Egg:=Egg+'O';
         4:if(Key=ord('N'))then
               begin
               MessageDlg('Version .95, Created by '+Egg+'N', mtInformation, [mbOk], 0);
               end;

         else
            Egg:='';
         end;{case}
      end;{if}

end;

procedure TDLGLocate.txtSearchStringChange(Sender: TObject);
begin
   if(Length(txtSearchString.Text)>0)then
      bOK.Enabled:=True
   else
      bOK.Enabled:=False;
end;

procedure TDLGLocate.bHelpClick(Sender: TObject);
var
   pString: PChar;
begin
     pString:=StrAlloc(255);
     StrPCopy(pString, HelpFile);
     WinHelp(Application.Handle, pString, HELP_CONTENTS, 0);
     StrDispose(pString);
end;

end.

(*
// ==============================================================
// Copyright (c) 1996 by Aaron Castro and VK Thakur. All rights reserved
// Version 1.00 (Beta)   Date: 01/01/96
*)

